import type { MapObject } from '@/application/types'
import Dexie from 'dexie'

export class MapObjectStorage {
  private dexie: Dexie

  constructor() {
    this.dexie = new Dexie('map_objects')
    this.dexie.version(1).stores({
      map_objects: 'id, name, createdAt, updatedAt'
    })
  }

  async add(mapObject: MapObject, overwrite = false) {
    try {
      // Check if map object already exists, don't add if it does and overwrite is false
      const existingMapObject = await this.get(mapObject.id)
      if (existingMapObject && !overwrite) return

      await this.dexie.table('map_objects').put({
        ...mapObject
      })
    } catch (error) {
      console.error(`Failed to add map object ${mapObject.id}:`, error)
    }
  }

  async get(id: string): Promise<MapObject | null> {
    try {
      const mapObject = await this.dexie.table('map_objects').get(id)
      return mapObject || null
    } catch (error) {
      console.error(`Failed to retrieve map object ${id}:`, error)
      return null
    }
  }

  async getAll(): Promise<MapObject[]> {
    try {
      return await this.dexie.table('map_objects').toArray()
    } catch (error) {
      console.error('Failed to retrieve all map objects:', error)
      return []
    }
  }

  async getByName(name: string): Promise<MapObject[]> {
    try {
      return await this.dexie.table('map_objects').where('name').equals(name).toArray()
    } catch (error) {
      console.error(`Failed to retrieve map objects with name ${name}:`, error)
      return []
    }
  }

  async delete(id: string) {
    try {
      await this.dexie.table('map_objects').delete(id)
    } catch (error) {
      console.error(`Failed to delete map object ${id}:`, error)
    }
  }

  async clear() {
    try {
      await this.dexie.table('map_objects').clear()
    } catch (error) {
      console.error('Failed to clear map objects storage:', error)
    }
  }

  async update(id: string, updates: Partial<MapObject>) {
    try {
      const mapObject = await this.get(id)
      if (!mapObject) return false

      const updatedMapObject = {
        ...mapObject,
        ...updates
      }
      await this.add(updatedMapObject)
      return true
    } catch (error) {
      console.error(`Failed to update map object ${id}:`, error)
      return false
    }
  }
}
